<?php
/**
 * Skrill Order Overview
 *
 * The Class for view skrill order additional information
 * Copyright (c) Skrill
 *
 * @class   skrillorderoverview
 * @package skrill/Classes
 * @located at /controllers/admin
 */
class SkrillOrderOverview extends SkrillOrderOverview_parent
{
    /**
     * Render template for order review
     *
     * @return string template path
     */
    public function render()
    {
        $return = parent::render();

        $orderId = $this->getEditObjectId();

        if ($orderId != "-1" && isset($orderId)) {
            $skrillTransactionId = self::getSkrillTransactionIdFromOrder($orderId);
            $paymentLog = self::getPaymentLogByOrderId($orderId);

            if (isset($paymentLog['mb_transaction_id'])) {
                $this->_aViewData['transaction_id'] = $paymentLog['mb_transaction_id'];
            } elseif (isset($skrillTransactionId)) {
                $this->_aViewData['transaction_id'] = $skrillTransactionId;
            }
            if (isset($paymentLog['payment_type'])) {
                if ($paymentLog['payment_type'] == 'NGP') {
                    $paymentLog['payment_type'] = 'OBT';
                }
                if ($paymentLog['payment_type'] == 'GCI') {
                    $paymentLog['payment_type'] = 'IDL';
                }
                $this->_aViewData['payment_type'] = $paymentLog['payment_type'];
            }
            if (isset($paymentLog['payment_instrument_country'])) {
                $this->_aViewData['payment_instrument_country'] =
                    $this->getCountryTitle($paymentLog['payment_instrument_country']);
            }
            if (isset($paymentLog['ip_country'])) {
                $this->_aViewData['ip_country'] =
                    $this->getCountryTitle($paymentLog['ip_country']);
            }
            if (isset($paymentLog['pay_from_email'])) {
                $this->_aViewData['pay_from_email'] = $paymentLog['pay_from_email'];
            }
        }

        return $return;
    }

    private function getCountryTitle($countryIsoCode)
    {
        if (strlen($countryIsoCode) == 3) {
            $countryIsoCode2 = SkrillOxCountry::getCountryIso2byIso3($countryIsoCode);
        } else {
            $countryIsoCode2 = $countryIsoCode;
        }
        $oCountry = oxNew('oxCountry');
        $sCountryId = $oCountry->getIdByCode($countryIsoCode2);

        $oLang = oxRegistry::getLang();
        $templateLanguage = $oLang->getTplLanguage();

        $oCountry->loadInLang($templateLanguage, $sCountryId);

        $sCountryTitle = $oCountry->oxcountry__oxtitle->value;

        return $sCountryTitle;
    }

    /**
     * Get skrill payment log
     *
     * @param  integer $orderId order id
     * @return array
     */
    private function getPaymentLogByOrderId($orderId)
    {
        $sQuery = "SELECT `paymentresponse` FROM `skrill__payment_logs` WHERE `oxorderid` = '";
        if (SkrillCore::isOxidSix()) {
            $sQuery .= $orderId."'";
        } else {
            $sQuery .= oxDb::getInstance()->escapeString($orderId)."'";
        }
        $paymentLog = oxDb::getDB()->getone($sQuery);

        return json_decode($paymentLog, true);
    }

    /**
     * Get skrill transaction id from order
     *
     * @param  integer $orderId order id
     * @return string
     */
    private function getSkrillTransactionIdFromOrder($orderId)
    {
        $sQuery = "SELECT `skrill__transaction_id` FROM `oxorder` WHERE `OXID` = '";
        if (SkrillCore::isOxidSix()) {
            $sQuery .= $orderId."'";
        } else {
            $sQuery .= oxDb::getInstance()->escapeString($orderId)."'";
        }
        $transactionId = oxDb::getDB()->getone($sQuery);

        return $transactionId;
    }
}
