<?php
/**
 * Skrill Plugin Setup
 *
 * The Class for Process Skrill Payment Gateways
 * Copyright (c) Skrill
 *
 * @class   SkrillSetting
 * @package Skrill/Classes
 * @located at /controllers/admin
 */
class SkrillSetting extends Shop_Config
{

    const SKRILL__MODULE_NAME = 'skrill';

    /**
    * Current Version String.
     *
    * @var string
    */
    protected $_sVersion       = 'Skrill Payments Module v1.0.11';

    /**
    * Current class template.
     *
    * @var string
    */
    protected $_sThisTemplate  = 'SkrillSetting.tpl';

    protected $skrill_general_merchant_id = '';
    protected $skrill_general_merchant_account_email = '';
    protected $skrill_general_recipient = '';
    protected $skrill_general_logo_url = '';
    protected $skrill_general_api_password = '';
    protected $skrill_general_secret_password = '';

    protected $skrill_payment_methods_default = array(
        "ACC" => array("enabled" => 1),
        "ACI" => array("enabled" => 1),
        "ADB" => array("enabled" => 1),
        "ALI" => array("enabled" => 1),
        "AOB" => array("enabled" => 1),
        "BLK" => array("enabled" => 1),
        "FLEXIBLE" => array("enabled" => 1),
        "GIR" => array("enabled" => 1),
        "IDL" => array("enabled" => 1),
        "MAE" => array("enabled" => 1),
        "MBW" => array("enabled" => 1),
        "MSC" => array("enabled" => 1),
        "MUB" => array("enabled" => 1),
        "NTL" => array("enabled" => 1),
        "OBT" => array("enabled" => 1),
        "PCH" => array("enabled" => 1),
        "PGF" => array("enabled" => 1),
        "PLI" => array("enabled" => 1),
        "PSC" => array("enabled" => 1),
        "PWY" => array("enabled" => 1),
        "RER" => array("enabled" => 1),
        "SFT" => array("enabled" => 1),
        "SPX" => array("enabled" => 1),
        "VSA" => array("enabled" => 1),
        "WLT" => array("enabled" => 1)
    );

    protected $skrill__is_module_installed = null;

    /**
     * Initialize class
     *
     * @return string template path
     */
    public function init()
    {
        $this->_aViewData['skrill__sqlExecutionErrors'] = '';

        $this->skrill__oxid_payment_ids = SkrillCore::getOxidPaymentIds();

        return parent::init();
    }

    /**
    * RENDER -------------------------------------------------------------
    */

    /**
    * Executes parent method parent::render() and returns name of template
    * file "skrill.tpl".
    *
    * @return string
    */
    public function render()
    {
        $save_status = '';
        if (!empty(oxRegistry::getSession()->getVariable('oxid_save_setting'))) {
            $save_status = oxRegistry::getSession()->getVariable('oxid_save_setting');
            oxRegistry::getSession()->deleteVariable('oxid_save_setting');
            $this->_aViewData['active_tab'] = 'setting';
        } elseif (!empty(oxRegistry::getSession()->getVariable('oxid_save_payment'))) {
            $save_status = oxRegistry::getSession()->getVariable('oxid_save_payment');
            oxRegistry::getSession()->deleteVariable('oxid_save_payment');
            $this->_aViewData['active_tab'] = 'payment';
        }
        $this->_aViewData['save_status'] = $save_status;

        $oDb = oxDb::getDb();

        $myConfig = $this->getConfig();
        parent::render();

        $sCurrentAdminShop = $myConfig->getShopId();
        $query = "SELECT * FROM oxconfig WHERE OXVARNAME = 'skrill_term'";
        $skrillTermResult = SkrillDb::getInstance()->select($query);

        if ($skrillTermResult->count() == 0) {
            $sql = "INSERT INTO `oxconfig` (`OXID`, `OXSHOPID`, `OXVARNAME`, `OXVARTYPE`, `OXVARVALUE`) values (";
            $sql .= $oDb->quote(oxUtilsObject::getInstance()->generateUID()).", '".$sCurrentAdminShop;
            $sql .= "', 'skrill_term', 'bool', '1')";
            $this->executeSql($sql);
            $this->_aViewData['skrill_term'] = 'True';
        }

        $this->setViewdataPaymentConfiguration();
        $this->setTooltopsPaymentConfiguration();
        $this->_aViewData['skrill_payment_methods'] = $this->definePaymentMethods();
        $this->_aViewData['languages'] = $this->getMultiLanguage();
        $this->_aViewData['bottom_buttons'] = 'test';

        // module isn't installed
        if ($this->isModuleSkrillInstalled()) {
            $this->_aViewData['start_setup'] = true; // if module running in producitve mode -> linking to prod account
        } elseif (stristr($this->getConfig()->getConfigParam('skrill__gateway_url_redirect'), 'prod')) {
            $this->_aViewData['bottom_buttons'] = 'prod';
        }

        $this->setViewDataWithConfigVars($sCurrentAdminShop);

        $this->_aViewData['version'] = $this->_sVersion;
        // hidden default bottom custom navi
        $this->_aViewData['sHelpURL'] = false;
        $this->_aViewData['oxid'] = $sCurrentAdminShop;

        return $this->_sThisTemplate;
    }

    /**
     * Get multi language
     *
     * @return object
     */
    protected function getMultiLanguage()
    {

        $myConfig = $this->getConfig();
        $oLang      = oxRegistry::getLang();
        $iLanguage  = $oLang->getBaseLanguage();
        $tplLanguage  = $oLang->getTplLanguage();
        $aLanguages = $oLang->getLanguageArray($iLanguage);

        $this->_aViewData['lang_abbr'] = $aLanguages[$tplLanguage]->abbr;

        foreach ($aLanguages as $id => $data) {
            if ($aLanguages[$iLanguage]->selected === 1) {
                switch (true) {
                    case stristr($aLanguages[$iLanguage]->abbr, 'EN'):
                        $this->_aViewData['buttons_languages'] = '1';
                        break; // English
                    default:
                        $this->_aViewData['buttons_languages'] = '5';
                        break; // German
                };
            }

            $aLanguages[$id]->aliasUsage = $myConfig->getConfigParam('skrill_sAliasUsage' . $id);
            $aLanguages[$id]->tplTitle = $myConfig->getConfigParam('skrill_sTplTitle' . $id);
        }

        return $aLanguages;
    }

    /**
     * Define payment method
     *
     * @return array
     */
    protected function definePaymentMethods()
    {
        $skrillPaymentMethods = $this->getConfig()->getConfigParam('skrill_payment_methods');

        return $skrillPaymentMethods;
    }

    /**
     * Set viewdata for payment configuration page
     */
    protected function setViewdataPaymentConfiguration()
    {
        $this->_aViewData['skrill_general_merchant_id'] = $this->skrill_general_merchant_id;
        $this->_aViewData['skrill_general_merchant_account_email'] = $this->skrill_general_merchant_account_email;
        $this->_aViewData['skrill_general_recipient'] = $this->skrill_general_recipient;
        $this->_aViewData['skrill_general_logo_url'] = $this->skrill_general_logo_url;
        $this->_aViewData['skrill_general_api_password'] = $this->skrill_general_api_password;
        $this->_aViewData['skrill_general_secret_password'] = $this->skrill_general_secret_password;

        $this->_aViewData['skrill_module_version'] = (new SkrillConfig)->moduleVersion;

        $paymentMethods = SkrillCore::$paymentMethods;
        foreach ($paymentMethods as $paymentMethodCode => $paymentMethod) {
            if (is_array($paymentMethod['allowedCountries'])) {
                foreach ($paymentMethod['allowedCountries'] as $bankOfCountries) {
                    if (is_array($bankOfCountries)) {
                        foreach (array_keys($bankOfCountries) as $bankName) {
                            $paymentMethods[$paymentMethodCode]['banks'][] = $bankName;
                        }
                    }
                }
            }
        }

        $this->_aViewData['all_skrill_payment_methods'] = $paymentMethods;
        
        $this->_aViewData['skrill_payment_methods_default'] = $this->skrill_payment_methods_default;
    }

    /**
     * Set tooltips for payment configuration page
     */
    protected function setTooltopsPaymentConfiguration()
    {
        $skrillTooltips = array();
        $skrillTooltips['psc'] = 'Belgium, Croatia,
	    Cyprus, Czech Republic, Denmark, Estonia, Finland,
		Georgia, Gibraltar, Hungary, Ireland, Kuwait,
		Latvia, Liechtenstein, Lithuania, Luxembourg,
		Malta, Mexico, Netherlands, Norway, Peru, Portugal,
		Romania, Saudi Arabia, Slovakia, Slovenia, Sweden,
		Switzerland, Uruguay';
        $skrillTooltips['pch'] = 'Austria, Belgium, Bulgaria,
		Croatia, Cyprus, Czech Republic, France, Greece,
		Hungary, Ireland, Italy, Latvia, Lithuania, Luxembourg,
		Malta, Netherlands, Poland, Portugal, Romania, Slovakia,
		Slovenia, Spain, Sweden, Switzerland,
		United Kingdom, United States of America';
        $skrillTooltips['wlt'] = 'All Countries';
        $skrillTooltips['acc'] = 'All Countries';
        $skrillTooltips['vsa'] = 'All Countries';
        $skrillTooltips['msc'] = 'All Countries';
        $skrillTooltips['mae'] = 'United Kingdom, Spain, Ireland and Austria';
        $skrillTooltips['obt'] = 'Austria, Belgium, Denmark, Estonia, 
		Finland, France, Germany, Ireland, Italy, Latvia, Lithuania,
		Netherlands, Portugal, Romania, Spain, United Kingdom';
        $skrillTooltips['gir'] = 'Germany';
        $skrillTooltips['idl'] = 'Netherlands';
        $skrillTooltips['sft'] = 'Germany, Austria, Belgium, Netherlands,
		France, Hungary, Slovakia, Czech Republic, United Kingdom';
        $skrillTooltips['pli'] = 'Australia';
        $skrillTooltips['pwy'] = 'Poland';

        $aliTooltip = oxRegistry::getLang()->translateString('SKRILL_BACKEND_TT_ALI');
        if ($aliTooltip == "SKRILL_BACKEND_TT_ALI") {
            $skrillTooltips['ali'] = 'Consumer location: China only.
                Merchant location: This is available for merchants in all countries except China.';
        } else {
            $skrillTooltips['ali'] = $aliTooltip;
        }

        $skrillTooltips['ntl'] = "All except for
            Afghanistan, Armenia, Bhutan, Bouvet Island,
            Myanmar, China, (Keeling) Islands, Democratic
            Republic of Congo, Cook Islands, Cuba, Eritrea,
            South Georgia and the South Sandwich Islands,
            Guam, Guinea, Territory of Heard Island and
            McDonald Islands, Iran, Iraq, Cote d'Ivoire,
            Kazakhstan, North Korea, Kyrgyzstan, Liberia,
            Libya, Mongolia, Northern Mariana Islands,
            Federated States of Micronesia, Marshall
            Islands, Palau, Pakistan, East Timor, Puerto Rico,
            Sierra Leone, Somalia, Zimbabwe, Sudan, Syria,
            Tajikistan, Turkmenistan, Uganda, United
            States, US Virgin Islands, Uzbekistan, and Yemen";
        $skrillTooltips['adb'] = "Argentina, Brazil";
        $skrillTooltips['aob'] = "Brazil, Chile, China, Columbia";
        $skrillTooltips['aci'] = "Argentina, Brazil, Chile, China, Columbia,
            Mexico, Peru, Uruguay";
        $skrillTooltips['blk'] = 'Poland';
        $skrillTooltips['mbw'] = 'Portugal';
        $skrillTooltips['mub'] = 'Portugal';
        $skrillTooltips['pgf'] = 'Peru';
        $skrillTooltips['rer'] = 'Indonesia, Philippines';
        $skrillTooltips['spx'] = 'Brazil';

        $this->_aViewData['skrill_tooltips'] = $skrillTooltips;
    }


    /**
     * Execute Sql
     *
     * @param string $sql sql query
     */
    protected function executeSql($sql)
    {
        $result = oxDb::getDb()->Execute($sql);
        if ($result === false) {
            // query could not executed
            $this->_aViewData['skrill__sqlExecutionErrors'] .= $sql . ";\n";
        }
    }

    /**
     * Get language key for oxid payment id
     *
     * @param  [type] $oxidPaymentId [description]
     * @return [type]                [description]
     */
    protected function getLanguageKeyForOxidPaymentId($oxidPaymentId)
    {
        $value = explode("_", $oxidPaymentId, 2);
        $extension = strtoupper(end($value));
        $langKey = "SKRILL_BACKEND_PM_" . $extension;
        return $langKey;
    }

    /**
     * Check if module skrill already installed
     *
     * @return boolean
     */
    protected function isModuleSkrillInstalled()
    {
        if ($this->skrill__is_module_installed !== null) {
            return $this->skrill__is_module_installed;
        }

        $shopId = $this->getConfig()->getShopId();
        $oxDb   = oxDb::getDb();

        $sql = "SELECT OXID
                FROM oxconfig
                WHERE
                OXVARNAME LIKE  '%skrill%'
                AND OXSHOPID = '" .$shopId . "'
                LIMIT 1";
        return $this->skrill__is_module_installed = (bool) $oxDb->getOne($sql);
    }

    /**
     * Get config variable from database
     *
     * @param  string $shopId shop id
     * @return array
     */
    protected function getConfigVariableFromDB($shopId)
    {
        $shopHeadVersion = oxRegistry::getConfig()->getVersion();
        if (version_compare($shopHeadVersion, '4.7.0', '<=')) {
            return $this->_loadConfVars($shopId, $moduleName   = '');
        }
        return $this->loadConfVars($shopId, $moduleName   = '');
    }

    /**
     * Set viewdata with config vars
     *
     * @param string $shopId shop id
     */
    protected function setViewDataWithConfigVars($shopId)
    {
        $aDbVariables = $this->getConfigVariableFromDB($shopId);
        $aConfVars    = $aDbVariables['vars'];

        // $sParam:$sType = confbools:bool // confstrs:str // confarrs:arr
        // confaarrs:aarr// confselects:select // confnum:num
        foreach ($this->_aConfParams as $sType => $sParam) {
            $this->_aViewData[$sParam] = $aConfVars[$sType];
        }
    }

    /**
    * Saves skrill general setting.
    */
    public function saveSetting()
    {
        try {
            $this->saveConfVars();
            oxRegistry::getSession()->setVariable('oxid_save_setting', 'success');
        } catch (Exception $e) {
            oxRegistry::getSession()->setVariable('oxid_save_setting', 'failed');
        }
    }

    /**
    * Saves skrill payment configuration.
    */
    public function savePaymentConfiguration()
    {
        try {
            $this->updateSkrillPayments();
            oxRegistry::getSession()->setVariable('oxid_save_payment', 'success');
        } catch (Exception $e) {
            oxRegistry::getSession()->setVariable('oxid_save_payment', 'failed');
        }
    }

    /**
    * Install new tables, update fields and clear cache
    */
    protected function updateSkrillPayments()
    {
        // Save skrill payment to show on frontend select payment
        $this->installSkrillPayments();

        // Save skrill options on config
        $skrillPaymentMethods = oxRegistry::getConfig()->getRequestParameter("skrill_payment_methods");
        $this->saveShopConfFromArray(array('skrill_payment_methods' => $skrillPaymentMethods), 'aarr', true);
    }

    /**
     * Save config variables
     */
    public function saveConfVars()
    {
        $myConfig = $this->getConfig();
        $aConfBools = oxRegistry::getConfig()->getRequestParameter("confbools");
        $aConfStrs  = oxRegistry::getConfig()->getRequestParameter("confstrs");
        $aConfArrs  = oxRegistry::getConfig()->getRequestParameter("confarrs");
        $aConfAarrs = oxRegistry::getConfig()->getRequestParameter("confaarrs");

        if ($aConfStrs['skrill_general_api_password'] == '*****') {
            unset($aConfStrs['skrill_general_api_password']);
        } else {
            $aConfStrs['skrill_general_api_password'] = md5($aConfStrs['skrill_general_api_password']);
        }
        if ($aConfStrs['skrill_general_secret_password'] == '*****') {
            unset($aConfStrs['skrill_general_secret_password']);
        } else {
            $aConfStrs['skrill_general_secret_password'] = md5($aConfStrs['skrill_general_secret_password']);
        }

        // special case for min order price value
        if ($aConfStrs['iMinOrderPrice']) {
            $aConfStrs['iMinOrderPrice'] = str_replace(',', '.', $aConfStrs['iMinOrderPrice']);
        }

        $this->saveShopConfFromArray($aConfBools, 'bool');

        $this->saveShopConfFromArray($aConfStrs, 'str');

        // home country multiple selectlist feature
        $this->saveShopConfFromArray($aConfArrs, 'arr', true);

        $this->saveShopConfFromArray($aConfAarrs, 'aarr', true);
    }

    /**
     * Save shop config from array
     *
     * @param array   $confVars           configuration variables
     * @param string  $varKey             variable configuration key
     * @param boolean $isMultilineToArray if value need to change into array from multiline
     */
    protected function saveShopConfFromArray($confVars, $varKey, $isMultilineToArray = false)
    {
        $myConfig = $this->getConfig();
        if (is_array($confVars)) {
            foreach ($confVars as $sVarName => $aVarVal) {
                if ($isMultilineToArray and !is_array($aVarVal)) {
                    $aVarVal = $this->_multilineToAarray($aVarVal);
                }
                $myConfig->saveShopConfVar($varKey, $sVarName, $aVarVal);
            }
        }
    }

    /**
     * Get skrill payment methods from request
     *
     * @return [type] [description]
     */
    protected function getSkrillPaymentMethodsFromRequest()
    {
        $paymentMethodParameters = oxRegistry::getConfig()->getRequestParameter("skrill_payment_methods");
        $isCCSenabled = $paymentMethodParameters['ACC']['enabled'] == 1;
        $separatelyPaymentMethods = array('VSA', 'MSC', 'MAE');
        $skrillPaymentMethods = array();

        foreach ($paymentMethodParameters as $paymentMethod => $options) {
            if (in_array($paymentMethod, $separatelyPaymentMethods) && $isCCSenabled) {
                continue;
            }
            
            if ($options['enabled']) {
                $skrillPaymentMethods["skrill_" . $paymentMethod] = $options;
            }
        }
        return $skrillPaymentMethods;
    }

    /**
     * Install skrill payments
     */
    protected function installSkrillPayments()
    {
        $enabledPaymentMethods = $this->getSkrillPaymentMethodsFromRequest();
        $aLanguageParams = array_values($this->getConfig()->getConfigParam('aLanguageParams'));

        foreach ($this->skrill__oxid_payment_ids as $oxidPaymentId) {
            $oResult = SkrillDb::getInstance()->select(
                "SELECT * FROM oxpayments WHERE OXID = '" .$oxidPaymentId. "'"
            );

            // add payment method to oxpayments
            if ($oResult->count() == 0 and isset($enabledPaymentMethods[$oxidPaymentId])) {
                // add oxpayment entry
                $sQuery = "INSERT INTO `oxpayments` (`OXID`, `OXACTIVE`, `OXTOAMOUNT`, ";

                $queryLanguageColumnNames = '';
                foreach ($aLanguageParams as $aLanguageParam) {
                    if (!empty($queryLanguageColumnNames)) {
                        $queryLanguageColumnNames .= ", ";
                    }

                    if ($aLanguageParam["baseId"] > 0) {
                        $queryLanguageColumnNames .= "`OXDESC_" .$aLanguageParam["baseId"]. "` ";
                    } else {
                        $queryLanguageColumnNames .= "`OXDESC` ";
                    }
                }
                $sQuery .= $queryLanguageColumnNames;
                $sQuery .= ") VALUES ('" .$oxidPaymentId . "', 1, 1000000, ";

                $langKey = $this->getLanguageKeyForOxidPaymentId($oxidPaymentId);
                $oxLang  = oxRegistry::getLang();

                $queryLanguageColumnValues = '';
                foreach ($aLanguageParams as $aLanguageParam) {
                    if (!empty($queryLanguageColumnValues)) {
                        $queryLanguageColumnValues .= ', ';
                    }

                    $queryLanguageColumnValues .=
                          "'".$oxLang->translateString(
                              $langKey,
                              $aLanguageParam['baseId']
                          )."'";
                }
                $sQuery .= $queryLanguageColumnValues;

                $sQuery .= ")";
                oxDb::getDb()->Execute($sQuery);
            } elseif ($oResult->count() != 0 and !isset($enabledPaymentMethods[$oxidPaymentId])) {
                // delete payment method from oxpayments
                $payment = oxNew('oxpayment');
                $payment->load($oxidPaymentId);
                $payment->delete();
            }
        }
    }

    /**
     * get database name that already escaped as srtring type data
     * @return string
     */
    protected function getEscapeDbName()
    {
        global $ADODB_SESSION_DRIVER;
        $dbName    = oxRegistry::getConfig()->getConfigParam('dbName');
        $dbEscapeName = $dbName;

        return $dbEscapeName;
    }
}
