<?php
/**
* 2015 Skrill
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
*
*  @author    Skrill <contact@skrill.com>
*  @copyright 2015 Skrill
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of Skrill
*/

class SkrillCore
{
    const PROCESSEDSTATUS = '2';
    const PENDINGSTATUS = '0';
    const FAILEDSTATUS = '-2';
    const REFUNDEDSTATUS = '-4';
    const REFUNDFAILEDSTATUS = '-5';
    const REFUNDPENDINGSTATUS = '-6';
    const INVALIDCREDENTIALSTATUS = '-7';
    const FRAUDSTATUS = '-8';

    /**
     * @var skrillPrepareUrl
     */
    protected static $skrillPrepareUrl = 'https://pay.skrill.com';

    /**
     * @var skrillQueryUrl
     */
    protected static $skrillQueryUrl = 'https://www.skrill.com/app/query.pl';

    /**
     * @var skrillRefundUrl
     */
    protected static $skrillRefundUrl = 'https://www.skrill.com/app/refund.pl';

    /**
     * @var allowedCountries
     */
    public static $allowedCountries = array(
        'ALA','ALB','DZA','ASM','AND','AGO','AIA','ATA','ATG','ARG','ARM','ABW','AUS','AUT','AZE','BHS','BHR','BGD',
        'BRB','BLR','BEL','BLZ','BEN','BMU','BTN','BOL','BIH','BWA','BVT','BRA','BRN','BGR','BFA','BDI','KHM','CMR',
        'CAN','CPV','CYM','CAF','TCD','CHL','CHN','CXR','CCK','COL','COM','COG','COD','COK','CRI','CIV','HRV','CYP',
        'CZE','DNK','DJI','DMA','DOM','ECU','EGY','SLV','GNQ','ERI','EST','ETH','FLK','FRO','FJI','FIN','FRA','GUF',
        'PYF','ATF','GAB','GMB','GEO','DEU','GHA','GIB','GRC','GRL','GRD','GLP','GUM','GTM','GGY','HTI','HMD','VAT',
        'GIN','GNB','GUY','HND','HKG','HUN','ISL','IND','IDN','IRL','IMN','ISR','ITA','JAM','JPN','JEY','JOR','KAZ',
        'KEN','KIR','KOR','KWT','LAO','LVA','LBN','LSO','LBR','LIE','LTU','LUX','MAC','MKD','MDG','MWI','MYS','MDV',
        'MLI','MLT','MHL','MTQ','MRT','MUS','MYT','MEX','FSM','MDA','MCO','MNG','MNE','MSR','MAR','MOZ','MMR','NAM',
        'NPL','NLD','ANT','NCL','NZL','NIC','NER','NGA','NIU','NFK','MNP','NOR','OMN','PAK','PLW','PSE','PAN','PNG',
        'PRY','PER','PHL','PCN','POL','PRT','PRI','QAT','REU','ROU','RUS','RWA','SHN','KNA','LCA','MAF','SPM','VCT',
        'WSM','SMR','STP','SAU','SEN','SRB','SYC','SLE','SGP','SVK','SVN','SLB','SOM','ZAF','SGS','ESP','LKA','SUR',
        'SJM','SWZ','SWE','CHE','TWN','TJK','TZA','THA','TLS','TGO','TKL','TON','TTO','TUN','TUR','TKM','TCA','TUV',
        'UGA','UKR','ARE','GBR','USA','UMI','URY','UZB','VUT','VEN','VNM','VGB','VIR','WLF','ESH','YEM','ZMB','ZWE'
    );

    /**
     * @var unallowedCountries
     */
    public static $unallowedCountries = array(
        'AFG','CUB','ERI','IRN','IRQ','JPN','KGZ','LBY','PRK','SDN','SSD','SYR'
    );

    /**
     * @var paymentMethods
     */
    public static $paymentMethods = array (
        'FLEXIBLE' => array(
            'name' => 'Pay By Skrill',
            'allowedCountries' => 'ALL'
        ),
        'WLT' => array(
            'name' => 'Skrill Wallet',
            'allowedCountries' => 'ALL'
        ),
        'ACC' => array(
            'name' => 'Credit Card / Visa, Mastercard',
            'allowedCountries' => 'ALL'
        ),
        'VSA' => array(
            'name' => 'Visa',
            'allowedCountries' => 'ALL'
        ),
        'MSC' => array(
            'name' => 'MasterCard',
            'allowedCountries' => 'ALL'
        ),
        'MAE' => array(
            'name' => 'Maestro',
            'allowedCountries'  => array('GBR','ESP','IRL','AUT')
        ),
        'PSC' => array(
            'name' => 'Paysafecard',
            'allowedCountries' => array(
                'BEL', 'HRV', 'CYP', 'CZE', 'DNK', 'EST', 'FIN', 'GEO', 'GIB', 'HUN', 'IRL', 'KWT', 'LVA', 'LIE', 'LTU',
				'LUX', 'MLT', 'MEX', 'NLD', 'NOR', 'PER', 'PRT', 'ROU', 'SAU', 'SVK', 'SVN', 'SWE', 'SWZ', 'URY'
            )
        ),
        'PCH' => array(
            'name' => 'Paysafecash',
            'allowedCountries' => array(
                'AUT', 'BEL', 'BGR', 'HRV', 'CYP', 'CZE', 'FRA', 'GRC', 'HUN', 'IRL', 'ITA', 'LVA', 'LTU',
				'LUX', 'MLT', 'NLD', 'POL', 'PRT', 'ROU', 'SVK', 'SVN', 'ESP', 'SWE', 'SWZ', 'GBR', 'USA'
            )
        ),
        'OBT' => array(
            'name' => 'Rapid Transfer',
            'allowedCountries'  => array(
                'AUT', 'BEL', 'DNK', 'EST', 'FIN', 'FRA', 'DEU', 'IRL', 'ITA', 'LVA', 'LTU', 'NLD', 'PRT', 'ROU', 'ESP', 'GBR'
            )
        ),
        'GIR' => array(
            'name' => 'Giropay',
            'allowedCountries' => array('DEU')
        ),
        'IDL' => array(
            'name' => 'iDEAL',
            'allowedCountries' => array('NLD')
        ),
        'SFT' => array(
            'name' => 'Sofort',
            'allowedCountries' => array('DEU', 'AUT', 'BEL', 'NLD', 'FRA', 'HUN', 'SVK', 'CZE', 'GBR')
        ),
        'PLI' => array(
            'name' => 'POLi',
            'allowedCountries' => array('AUS')
        ),
        'PWY' => array(
            'name' => 'Przelewy24',
            'allowedCountries' => array('POL')
        ),
        'ALI' => array(
            'name' => 'Alipay',
            'allowedCountries' => array('CHN')
        ),
        'NTL' => array(
            'name' => 'Neteller',
            'allowedCountries'  => 'ALL',
            'exceptedCountries' => array(
                'AFG','ARM','BTN','BVT','MMR','CHN','COD','COK','CUB','ERI','SGS','GUM','GIN','HMD','IRN','IRQ','CIV',
                'KAZ','PRK','KGZ','LBR','LBY','MNG','MNP','FSM','MHL','PLW','PAK','TLS','PRI','SLE','SOM','ZWE','SDN',
                'SYR','TJK','TKM','UGA','USA','VIR','UZB','YEM'
            )
        ),
        'BLK' => array(
            'name' => 'Blik',
            'allowedCountries' => array('POL')
        ),
        'MBW' => array(
            'name' => 'MB WAY',
            'allowedCountries' => array('PRT')
        ),
        'MUB' => array(
            'name' => 'Multibanco',
            'allowedCountries' => array('PRT')
        ),
        'PGF' => array(
            'name' => 'PagoEfectivo',
            'allowedCountries' => array('PER')
        ),
        'RER' => array(
            'name' => 'Rapyd E-wallet',
            'allowedCountries' => array('IDN','PHL')
        ),
        'SPX' => array(
            'name' => 'PIX',
            'allowedCountries' => array('BRA')
        ),
        'ACI' => array(
            'name' => 'Cash / Invoice',
            'allowedCountries' => array(
                'ARG' => array(
                    'RedLink' => 'red-link.png',
                    'Pago Facil' => 'pago-facil.png'
                ),
                'BRA' => array(
                    'Boleto Bancario' => 'boleto-bancario.png'
                ),
                'CHL' => array(
                    'Servi Pag' => 'servi-pag.png'
                ),
                'COL' => array(
                    'Efecty' => 'efecty.png',
                    'Davivienda' => 'davivienda.png',
                    'Éxito' => 'exito.png',
                    'Carulla' => 'carulla.png',
                    'EDEQ' => 'edeq.png',
                    'SurtiMax' => 'surtimax.png'
                ),
                'MEX' => array(
                    'OXXO' => 'oxxo.png',
                    'BBVA Bancomer' => 'bancomer_m.png',
                    'Banamex' => 'banamex.png',
                    'Banco Santander' => 'santander.png'
                ),
                'PER' => array(
                    'Banco de Occidente' => 'banco-de-occidente.png'
                ),
                'URY' => array(
                    'Red Pagos' => 'red-pagos.png'
                )
            ),
            'backend_logos' => array('aci.png')
        ),
        'ADB' => array(
            'name' => 'Direct Bank Transfer',
            'allowedCountries' => array(
                'ARG' => array(
                    'Banco Santander Rio' => 'santander-rio.png'
                ),
                'BRA' => array(
                    'Banco Itau' => 'itau.png',
                    'Banco do Brasil' => 'banco-do-brasil.png',
                    'Banco Bradesco' => 'bradesco.png'
                )
            ),
            'backend_logos' => array('adb_.png'),
        ),
        'AOB' => array(
            'name' => 'Manual Bank Transfer',
            'allowedCountries' => array(
                'BRA' => array(
                    'HSBC' => 'hsbc.png',
                    'Caixa' => 'caixa.png',
                    'Santander' => 'santander.png'
                ),
                'CHL' => array(
                    'WebPay' => 'webpay.png'
                ),
                'COL' => array(
                    'PSEi' => 'PSEi.png',
                    'Bancolombia' => 'bancolombia.png'
                )
            ),
            'backend_logos' => array('aob.png'),
        )
    );

    /**
     * @var null|oxConfig
     */
    protected static $_oConfig = null;

    /**
     * Get all Skrill Payment Methods
     *
     * @return array
     */
    public static function getPaymentMethods()
    {
        return self::$paymentMethods;
    }

    /**
     * Get Skrill payment method from paymentMethods key
     *
     * @param string $paymentType
     * @return array
     */
    public static function getPaymentMethodByPaymentType($paymentType)
    {
        return self::$paymentMethods[$paymentType];
    }

    /**
     * Get Skrill Redirect Url by $skrillPrepareUrl and sid
     *
     * @param $sid
     * @return string
     */
    public static function getSkrillRedirectUrl($sid)
    {
        $skrillRedirectUrl = self::$skrillPrepareUrl.'?sid='.$sid;
        return $skrillRedirectUrl;
    }

    /**
     * Get Sid from Skrill gateway by request parameters
     *
     * @param array $fields
     * @return string
     */
    public static function getSid($fields)
    {
        $fields_string = http_build_query($fields);

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, self::$skrillPrepareUrl);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 2);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLINFO_HEADER_OUT, true);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-type: application/x-www-form-urlencoded;charset=UTF-8'));
        curl_setopt($curl, CURLOPT_FAILONERROR, 1);
        curl_setopt($curl, CURLOPT_POST, count($fields));
        curl_setopt($curl, CURLOPT_POSTFIELDS, $fields_string);

            $result = curl_exec($curl);
        if (curl_errno($curl)) {
            throw new Exception("Curl error: ". curl_error($curl));
        }
        curl_close($curl);

        return $result;
    }

    /**
     * try to get response from Skrill gateway three times
     *
     * @param array $fieldParams
     * @return boolean | array
     */
    public static function isPaymentAccepted($fieldParams)
    {
        // check status_trn 3 times if no response.
        for ($i=0; $i < 3; $i++) {
            $response = true;
            try {
                $result = self::doQuery('status_trn', $fieldParams);
            } catch (Exception $e) {
                $response = false;
            }
            if ($response && $result) {
                return self::getResponseArray($result);
            }
        }
        return false;
    }

    /**
     * Send request to Skrill gateway by action and request parameters
     *
     * @param string $action
     * @param array $fieldParams
     * @return array
     */
    public static function doQuery($action, $fieldParams)
    {
        $fieldType = $fieldParams['type'];
        $fields = array();
        $fields[$fieldType] = $fieldParams['id'];
        $fields['action'] = $action;
        $fields['email'] = $fieldParams['email'];
        $fields['password'] = $fieldParams['password'];

        $fields_string = http_build_query($fields);

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, self::$skrillQueryUrl);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 2);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLINFO_HEADER_OUT, true);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-type: application/x-www-form-urlencoded;charset=UTF-8'));
        curl_setopt($curl, CURLOPT_FAILONERROR, 1);
        curl_setopt($curl, CURLOPT_POST, count($fields));
        curl_setopt($curl, CURLOPT_POSTFIELDS, $fields_string);

        $result = curl_exec($curl);
        if (curl_errno($curl)) {
            throw new Exception("Curl error: ". curl_error($curl));
        }
        curl_close($curl);

        return $result;
    }

    /**
     * Send refund request to Skrill gateway by action and request parameters
     *
     * @param string $action
     * @param array $fieldParams
     * @return array
     */
    public static function doRefund($action, $fieldParams)
    {

        if ($action == "prepare") {
            $fields = $fieldParams;
            $fields['action'] = $action;
        } elseif ($action == "refund") {
            $fields['action'] = $action;
            $fields['sid'] = $fieldParams;
        }

        $fields_string = http_build_query($fields);

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, self::$skrillRefundUrl);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 2);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLINFO_HEADER_OUT, true);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-type: application/x-www-form-urlencoded;charset=UTF-8'));
        curl_setopt($curl, CURLOPT_FAILONERROR, 1);
        curl_setopt($curl, CURLOPT_POST, count($fields));
        curl_setopt($curl, CURLOPT_POSTFIELDS, $fields_string);

        $result = curl_exec($curl);
        if (curl_errno($curl)) {
              return "failed";
        }
        curl_close($curl);

        return simplexml_load_string($result);
    }

    /**
     * get Skrill Response in Array
     *
     * @param string $strings skrill response in string
     * @return array | boolean
     */
    public static function getResponseArray($strings)
    {
        $response_array = array();
        $string = explode("\n", $strings);
        $response_array['response_header'] = $string[0];
        if (!empty($string[1])) {
            $string_arr = explode("&", $string[1]);
            foreach ($string_arr as $value) {
                $value_arr = explode("=", $value);
                $response_array[urldecode($value_arr[0])] = urldecode($value_arr[1]);
            }
            return $response_array;
        }
        return false;
    }

    /**
     * get Date Time with format (ymdhiu)
     *
     * @return string
     */
    public static function getDateTime()
    {
            $t = microtime(true);
            $micro = sprintf("%06d", ($t - floor($t)) * 1000000);
            $d = new DateTime(date('Y-m-d H:i:s.'.$micro, $t));

            return $d->format("ymdhiu");
    }

    /**
     * Get Random Number by length
     *
     * @param int $length
     * @return int
     */
    public static function randomNumber($length)
    {
        $result = '';

        for ($i = 0; $i < $length; $i++) {
              $result .= mt_rand(0, 9);
        }

        return $result;
    }

    /**
     * Get Payment Status by code
     *
     * @param string $code
     * @return string
     */
    public static function getPaymentStatusByCode($code)
    {
        switch ($code) {
            case '2':
                $status = 'Payment Accepted';
                break;
            case '0':
                $status = 'Pending';
                break;
            case '-1':
                $status = 'Cancelled';
                break;
            case '-2':
                $status = 'Failed';
                break;
            case '-3':
                $status = 'Chargeback';
                break;
            case '-4':
                $status = 'Refunded';
                break;
            case '-5':
                $status = 'Refund Failed';
                break;
            case '-6':
                $status = 'Refund Pending';
                break;
            case '-7':
                $status = 'Invalid Credential';
                break;
            case '-8':
                $status = 'Suspected Fraud';
                break;
            default:
                $status = 'Unknown';
                break;
        }
        return $status;
    }

    public static function generateMd5sig($paymentResponse)
    {
        $merchantId = SkrillConfig::getSetting('skrill_general_merchant_id');
        $transactionId = $paymentResponse['transaction_id'];
        $secretWord = strtoupper(SkrillConfig::getSetting('skrill_general_secret_password'));
        $mbAmount = $paymentResponse['mb_amount'];
        $mbCurrency = $paymentResponse['mb_currency'];
        $status = $paymentResponse['status'];
        $string = $merchantId.$transactionId.$secretWord.$mbAmount.$mbCurrency.$status;

        return strtoupper(md5($string));
    }

    public static function isPaymentSignatureEqualsGeneratedSignature($paymentSignature, $generatedSignature)
    {
        return ($paymentSignature == $generatedSignature);
    }

    public static function isPaymentResponseValid($paymentResponse)
    {
        $generatedMd5Sig = self::generateMd5sig($paymentResponse);
        $isPaymentSignatureEqualsGeneratedSignature =
            self::isPaymentSignatureEqualsGeneratedSignature(
                $paymentResponse['md5sig'],
                $generatedMd5Sig
            );

        if ($isPaymentSignatureEqualsGeneratedSignature) {
            return true;
        }
        return false;
    }

    /**
     * get request parameters from gateway
     * @return array
     */
    public static function getRequestParametersFromGateway()
    {
        $response = array();
        foreach ($_REQUEST as $parameter => $value) {
            $parameter = strtolower($parameter);
            $response[$parameter] = $value;
        }

        return $response;
    }

    public static function getPaymentLogById($oxid)
    {
        $sQuery = "SELECT * FROM `skrill__payment_logs` WHERE `OXID` = '";
        if (self::isOxidSix()) {
            $sQuery .= $oxid."'";    
        } else {
            $sQuery .= oxDb::getInstance()->escapeString($oxid)."'";
        }
        $paymentLog = oxDb::getDB(oxDb::FETCH_MODE_ASSOC)->getRow($sQuery);

        return $paymentLog;
    }

    /**
     * get oxid std class base on oxid version
     * @return object
     */
    public static function getOxidStdClass()
    {
        
        if (self::isOxidSix()) {
            $stdClass = new stdClass();
        } else {
            $stdClass = new oxStdClass();
        }
        
        return $stdClass;
    }

    public static function getPaymentResponseByTransactionId($transactionId)
    {
        $sQuery = "SELECT `paymentresponse` FROM `skrill__payment_logs` WHERE `transactionid` = '";
        if (self::isOxidSix()) {
            $sQuery .= $transactionId."'";    
        } else {
            $sQuery .= oxDb::getInstance()->escapeString($transactionId)."'";
        }

        $paymentLog = oxDb::getDB(oxDb::FETCH_MODE_ASSOC)->getRow($sQuery);

        return json_decode($paymentLog['paymentresponse'], true);
    }

    public static function getPaymentResponseFromGateway($transactionId)
    {
        $parameters = array();
        $parameters['email'] = SkrillConfig::getSetting('skrill_general_merchant_account_email');
        $parameters['password'] = SkrillConfig::getSetting('skrill_general_api_password');
        $parameters['type'] = 'trn_id';
        $parameters['id'] = $transactionId;

        return self::isPaymentAccepted($parameters);
    }

    /**
     * get object order by ID
     * @param  string $oxid
     * @return object
     */
    public static function getOrderById($oxid)
    {
        $oOrder = oxNew('oxorder');
        $oOrder->load($oxid);
        
        return $oOrder;
    }

    public static function addPaymentLog($oxid, $paymentResponse)
    {
        $order = self::getOrderById($oxid);

        $orderId = $order->oxorder__oxordernr->value;
        $orderStatus = SkrillCore::getPaymentStatusByCode($paymentResponse['status']);
        $paymentConfig = SkrillConfig::getPaymentConfig($order->oxorder__oxpaymenttype);

        $assignVariables = array(
            'oxorderid'         => $oxid,
            'orderdate'         => date('Y-m-d H:i:s', strtotime($order->oxorder__oxorderdate->value)),
            'transactionid'     => $paymentResponse['transaction_id'],
            'mbtransactionid'   => $paymentResponse['mb_transaction_id'],
            'billfname'         => $order->oxorder__oxbillfname->value,
            'billlname'         => $order->oxorder__oxbilllname->value,
            'orderid'           => $orderId,
            'amount'            => $order->oxorder__oxtotalordersum->value,
            'currency'          => $order->oxorder__oxcurrency->value,
            'mbamount'          => $paymentResponse['mb_amount'],
            'mbcurrency'        => $paymentResponse['mb_currency'],
            'paiddate'          => $order->oxorder__oxpaid->value,
            'paymenttype'       => $paymentConfig['name'],
            'paymentstatus'     => $orderStatus,
            'paymentresponse'   => json_encode($paymentResponse),
        );

        SkrillLogger::set()->info('assignVariables add payment log :' . json_encode($assignVariables));

        $skrillPaymentLog = oxNew('SkrillPaymentLog');
        $skrillPaymentLog->assign($assignVariables);

        $skrillPaymentLog->save();
    }

    public static function updatePaymentLog($oxid, $paymentResponse)
    {
        $order = self::getOrderById($oxid);
        $orderStatus = SkrillCore::getPaymentStatusByCode($paymentResponse['status']);

        $sQuery = "UPDATE `skrill__payment_logs` SET ";
        $sQuery .= "`PAIDDATE` = '" . $order->oxorder__oxpaid->value . "', ";
        $sQuery .= "`PAYMENTSTATUS` = '" . $orderStatus . "' ";
        $sQuery .= "WHERE `oxorderid` = '" . $oxid."'";

        SkrillLogger::set()->info('Update payment log Query: ' . json_encode($sQuery));
        
        oxDb::getDb()->Execute($sQuery);
    }

    /**
     * update existing order status
     * @param  string $oxid
     * @param  array $paymentResponse
     * @return void
     */
    public static function updateExistingOrderStatus($oxid, $paymentResponse)
    {
        $oDb = oxDb::getDb();
        $orderStatus = SkrillCore::getPaymentStatusByCode($paymentResponse['status']);
        $paymentDate = date("Y-m-d H:i:s", time());

        $sQuery = "UPDATE `oxorder` SET `OXTRANSSTATUS` = " . $oDb->quote($orderStatus) . ", ";
        if ($paymentResponse['status'] == self::PROCESSEDSTATUS) {
            $sQuery .= "`OXPAID` = " . $oDb->quote($paymentDate) . ", ";
        }
        $sQuery .= "`skrill__status` = " . $oDb->quote($orderStatus) . " ";
        $sQuery .= "WHERE `OXID` = " . $oDb->quote($oxid);

        SkrillLogger::set()->info('update existing order status query: '.json_encode($sQuery));

        $oDb->Execute($sQuery);
    }

    /**
     * get Error Locale identifier by code
     *
     * @param string $code
     * @return string
     */
    public static function getSkrillErrorMapping($code)
    {
            $error_messages = array(
                  '01' => 'SKRILL_ERROR_01',
                  '02' => 'SKRILL_ERROR_02',
                  '03' => 'SKRILL_ERROR_03',
                  '04' => 'SKRILL_ERROR_04',
                  '05' => 'SKRILL_ERROR_05',
                  '08' => 'SKRILL_ERROR_08',
                  '09' => 'SKRILL_ERROR_09',
                  '10' => 'SKRILL_ERROR_10',
                  '12' => 'SKRILL_ERROR_12',
                  '15' => 'SKRILL_ERROR_15',
                  '19' => 'SKRILL_ERROR_19',
                  '24' => 'SKRILL_ERROR_24',
                  '28' => 'SKRILL_ERROR_28',
                  '32' => 'SKRILL_ERROR_32',
                  '37' => 'SKRILL_ERROR_37',
                  '38' => 'SKRILL_ERROR_38',
                  '42' => 'SKRILL_ERROR_42',
                  '44' => 'SKRILL_ERROR_44',
                  '51' => 'SKRILL_ERROR_51',
                  '63' => 'SKRILL_ERROR_63',
                  '70' => 'SKRILL_ERROR_70',
                  '71' => 'SKRILL_ERROR_71',
                  '80' => 'SKRILL_ERROR_80',
                  '98' => 'SKRILL_ERROR_98',
                  '99' => 'SKRILL_ERROR_99_GENERAL'
            );

        if ($code) {
            return array_key_exists($code, $error_messages) ? $error_messages[$code] : 'SKRILL_ERROR_99_GENERAL';
        } else {
            return 'SKRILL_ERROR_99_GENERAL';
        }
    }

    /**
     * Get payment logo for frontend
     * @param  string $paymentId        skrill payment id
     * @param  string $userCountryCode  user country code
     * @return string                   logo html
     */
    public static function getFrontendLogoHtmlById($paymentId, $userCountryCode = '')
    {
        if (strpos("skrill_", $paymentId) == -1) {
            return "";
        }

        $paymentId = str_replace('skrill_', '', $paymentId);
        $imgSource = oxRegistry::getConfig()->getSslShopUrl().'modules/skrill/out/images/';

        $paymentConfig = self::$paymentMethods[ $paymentId ];

        if (!$paymentConfig) {
            return "";
        }

        if (! isset($paymentConfig['allowedCountries'][$userCountryCode])
            || ! is_array($paymentConfig['allowedCountries'][$userCountryCode])
        ) {
            $alt = $paymentConfig['name'];
            $logo = strtolower($paymentId) . ".png";
            $height = $paymentId == 'FLEXIBLE' ? 'height="50"' : 'height="30"';
            $html = '<img src="'.$imgSource.$logo.'" '.$height.' alt="'.$alt.'" title="'.$alt.
            '" style="margin-right: 10px; margin-bottom: 7px;" />';
            return $html;
        } else {
            $html = "";
            $bankOfCountries = $paymentConfig['allowedCountries'][$userCountryCode];
            foreach ($bankOfCountries as $bankName => $bankLogo) {
                if ($bankName) {
                    $alt = $bankName;
                } else {
                    $alt = $paymentConfig['name'];
                }
                $html .= '<img src="'.$imgSource.$bankLogo.'" height="30" alt="'.$alt.'" title="'.$alt.
                    '" style="margin-right: 10px; margin-bottom: 7px;"/>';
            }
            return $html;
        }
    }

    /**
     * Get array of skrill payment IDs for OXID
     * @return array
     */
    public static function getOxidPaymentIds()
    {
        $oxidPaymentIds = array_keys(self::$paymentMethods);

        $oxidPaymentIds = array_map(function ($v) {
            return "skrill_" . $v;
        }, $oxidPaymentIds);

        return $oxidPaymentIds;
    }

    /**
     * Set number format to "NNNNNN.DD", number with dot as decimal separator with 2 digit decimals
     * @param string $number
     */
    public static function setNumberFormat($number)
    {
        $number = (float) str_replace(',', '.', $number);
        return number_format($number, 2, '.', '');
    }

    /**
     * @return oxConfig
     */
    protected static function _getConfig()
    {
        if (self::$_oConfig === null) {
            self::$_oConfig = oxRegistry::getConfig();
        }

        return self::$_oConfig;
    }

    /**
     * Returns true if the shop version is greater than 6.0.
     *
     * @return bool
     */
    public static function isOxidSix()
    {
        return version_compare(self::_getConfig()->getActiveShop()->getFieldData('oxversion'), '6.0', '>=');
    }
}
